/*
    Classe CPoints
*/

#include <stdlib.h>
#include <stdio.h>
#include "CPoints.h"

CPoints::CPoints() {
    vet = 0;
    initialize();
}

CPoints::~CPoints() {
    if(vet)
       delete vet;
    clear();
}

void CPoints::initialize() {
    pointcount = 0;
    start = 0;
    end = 0;
    pointer = 0;
    pmark = 0;
    isMarked = false;
}

// limpa a lista encadeada de pontos
void CPoints::clear() {
    CVertex *p, *i;
    i = start;
    
    while(i) {
           p = i;
           i = i->next;
           if(p->data)
              delete p->data;
           if(p)
              delete p;
    }
    initialize();
}

// adiciona um ponto  lista encadeada
CCoord* CPoints::add(float x, float y, float z) {
    CCoord *c = new CCoord;
    CVertex *v = new CVertex;
    if(c && v) {
         c->x = x;
         c->y = y;
         c->z = z;
         c->jarvis = false;
         if(end) {
            end->next = v;
            end = v;
         } else {
            start = v;
            end = v;
         }
         end->data = c;
         end->next = 0;
         pointer = end;
         pointcount ++;
    }
    return c;
}

// remove um ponto da lista encadeada
void CPoints::remove(int index) {
    CVertex *v, *a;
    if(index < pointcount) {
        v = start;
        if(index == 0) {
           if(pointcount == 1) {
              end = 0;
              start = 0;
           } else {
              start = start->next;
           }
           delete v->data;
           delete v;
        } else {
          int i = 1;
          do {
             a = v;
             v = v->next;
             if(i == index) {
                  a->next = v->next;
                  delete v->data;
                  delete v;
                  break;
             }
             i++;
          } while( i < pointcount );
        }
        pointcount --;
    }
}
                  
// retorna um determinado elemento da lista encadeada
CCoord* CPoints::get(int index) {
    int i = 0;
    CCoord *c = first();
    while(c) {
        if(i == index) break;
        c = next();
    }
    return c;
}

// retorna o nmero de elementos na lista encadeada
int CPoints::count() {
    return pointcount;
}

// retorna o primeiro ponto da lista
CCoord* CPoints::first() {
   CCoord *c = 0;
   if(start) {
      pointer = start;
      c = pointer->data;
   }
   return c;
}

// retorna o ltimo ponto da lista
CCoord* CPoints::last() {
   CCoord *c = 0;
   if(end) {
      pointer = end;
      c = pointer->data;
   }
   return c;
}

// retorna o prximo ponto da lista
CCoord* CPoints::next() {
   CCoord *c = 0;
   if(pointer->next) {
      pointer = pointer->next;
      c = pointer->data;
   }
   return c;
}

// retorna o ponto atual da lista
CCoord* CPoints::get() {
   CCoord *c = 0;
   if(pointer) {
      c = pointer->data;
   }
   return c;
}

// marca o ponto atual na lista
void CPoints::mark() {
     pmark = pointer;
}

// volta para o ponto marcado da lista
void CPoints::restore() {
     if(pmark)
         pointer = pmark;
}

// retorna um vetor com os pontos ordenados por x e y
CCoord** CPoints::sort() {
     CCoord* c;
     int i = 0;
     
     if(pointcount) {
         if(vet) 
            delete vet;
         vet = new CCoord*[pointcount];
         c = first();
         while(c) {
             vet[i] = c;
             c = next();
             i ++;
         }
         internalSort(vet);
     }

     return vet;
}

void CPoints::internalSort(CCoord** vet) {
     int i, j;
     CCoord * c;
     for(i = 0; i < pointcount-1; i++) {
           for(j = i+1; j < pointcount; j++) {
                 if(vet[i]->x > vet[j]->x) {
                     c = vet[i];
                     vet[i] = vet[j];
                     vet[j] = c;
                 } else if(vet[i]->x == vet[j]->x) {
                     if(vet[i]->y > vet[j]->y) {
                        c = vet[i];
                        vet[i] = vet[j];
                        vet[j] = c;
                     }
                 }
           }
     }
}


// -------------------------------------------------------------
// cdigo de carregamento dos pontos
// -------------------------------------------------------------

// carrega via um arquivo texto simples
// delimitado por ponto e vrgula
void CPoints::loadFromFile(char * fileName) {
     int linhas, i;
     FILE *infile;
     float x, y;
     
     clear();
     
     infile = fopen( fileName, "r");
     if(infile) {
         while(!feof(infile)) {
             fscanf(infile, "%f;%f\n", &x, &y);
             add(x, y, 0);
         }
         fclose(infile);
     }     
     
}

// carrega usando valores aleatrios
void CPoints::loadFromRandom(int size) {
     float x, y, z;
     clear();
     
   
     
     for(int i = 0; i < size; i++) {
             x = (rand()%1000) / 10.0;
             y = (rand()%1000) / 10.0;
             z = 0; //(rand()%1000) / 10.0;
             add(x, y, z);
     }
}

